#****************************************************************************
#**
#**  File     : /lua/wreckage.lua
#**
#**  Summary  : Class for wreckage so it can get pushed around
#**
#**  Copyright 2006 Gas Powered Games, Inc.  All rights reserved.
#****************************************************************************
local Prop = import('/lua/sim/Prop.lua').Prop
local allWrecks = {}
local idNum = 1
allWrecks.num = 0

DecayThread = function()
    while 1 do
        #LOG("wrecks = " .. allWrecks.num)

        for wreck, i in allWrecks do
            if not IsDestroyed(wreck) then wreck:DoDecayDamage(10) end
        end
        WaitTicks(100)
    end
end

ForkThread(DecayThread)

Wreckage = Class(Prop) {
    id = 0,
    OnCreate = function(self)
        Prop.OnCreate(self)
        idNum = idNum + 1
        self.id = idNum
        allWrecks[self] = self.id
        allWrecks.num = allWrecks.num + 1
    end,
    
    OnDamage = function(self, instigator, amount, vector, damageType)
        self:DoTakeDamage(instigator, amount, vector, damageType)
    end,

    DoTakeDamage = function(self, instigator, amount, vector, damageType)
        local maxHealth = self:GetMaxHealth()
        self:AdjustHealth(instigator, -amount)
        local health = self:GetHealth()
        if health > 0 then
            local healthRatio = health / maxHealth
            if not self.MaxReclaimTimeMassMult then
                self.MaxReclaimTimeMassMult = self.ReclaimTimeMassMult
            end
            if not self.MaxReclaimTimeEnergyMult then
                self.MaxReclaimTimeEnergyMult = self.ReclaimTimeEnergyMult
            end
            local mtime = self.MaxReclaimTimeMassMult * healthRatio
            local etime = self.MaxReclaimTimeEnergyMult * healthRatio
            local mass = self.MaxMassReclaim * healthRatio
            local energy = self.MaxEnergyReclaim * healthRatio
            self:SetReclaimValues( mtime, etime, mass, energy)
        elseif health <= 0 then
            ##LOG("DESTROYING: " .. self.id)
            allWrecks[self] = nil
            allWrecks.num = allWrecks.num - 1
            self:Destroy()

        end
    end,
   
    DoDecayDamage = function(self, amount)
        local maxHealth = self:GetMaxHealth()
        self:AdjustHealth(nil, -amount)
        local health = self:GetHealth()
        ##LOG("wreck health = " .. health)
        if health > 0 then
            local healthRatio = health / maxHealth
            if not self.MaxReclaimTimeMassMult then
                self.MaxReclaimTimeMassMult = self.ReclaimTimeMassMult
            end
            if not self.MaxReclaimTimeEnergyMult then
                self.MaxReclaimTimeEnergyMult = self.ReclaimTimeEnergyMult
            end
            local mtime = self.MaxReclaimTimeMassMult * healthRatio
            local etime = self.MaxReclaimTimeEnergyMult * healthRatio
            local mass = self.MaxMassReclaim * healthRatio
            local energy = self.MaxEnergyReclaim * healthRatio
            self:SetReclaimValues( mtime, etime, mass, energy)
        elseif health <= 0 then
            ##LOG("DESTROYING: " .. self.id)
            allWrecks[self] = nil
            allWrecks.num = allWrecks.num - 1
            self:Destroy()
            return true
        end
        return nil
    end,

    OnCollisionCheck = function(self, other)
        if IsUnit(other) then
            return false
        else
            return true
        end
    end,
}
